import type { AnyObject, Callback, ValidateOptions } from './types';
import type { ResolveOptions } from './Condition';
import type { CastOptions, SchemaFieldDescription, SchemaLazyDescription } from './schema';
import { Flags, ISchema } from './util/types';
import { BaseSchema } from '.';
export declare type LazyBuilder<T, TContext = AnyObject, TDefault = any, TFlags extends Flags = any> = (value: any, options: ResolveOptions) => ISchema<T, TContext, TFlags, TDefault>;
export declare function create<T, TContext = AnyObject, TFlags extends Flags = any, TDefault = any>(builder: LazyBuilder<T, TContext, TDefault, TFlags>): Lazy<T, TContext, TDefault, TFlags>;
export interface LazySpec {
    meta: Record<string, unknown> | undefined;
}
declare class Lazy<T, TContext = AnyObject, TDefault = any, TFlags extends Flags = any> implements ISchema<T, TContext, TFlags, TDefault> {
    private builder;
    type: "lazy";
    __isYupSchema__: boolean;
    readonly __outputType: T;
    readonly __context: TContext;
    readonly __flags: TFlags;
    readonly __default: TDefault;
    spec: LazySpec;
    constructor(builder: LazyBuilder<T, TContext, TDefault, TFlags>);
    clone(): Lazy<T, TContext, TDefault, TFlags>;
    private _resolve;
    resolve(options: ResolveOptions<TContext>): BaseSchema<T, TContext, TDefault, TFlags>;
    cast(value: any, options?: CastOptions<TContext>): T;
    validate(value: any, options?: ValidateOptions, maybeCb?: Callback): Promise<T>;
    validateSync(value: any, options?: ValidateOptions<TContext>): T;
    validateAt(path: string, value: any, options?: ValidateOptions<TContext>): Promise<any>;
    validateSyncAt(path: string, value: any, options?: ValidateOptions<TContext>): any;
    isValid(value: any, options?: ValidateOptions<TContext>): Promise<boolean>;
    isValidSync(value: any, options?: ValidateOptions<TContext>): boolean;
    describe(options?: ResolveOptions<TContext>): SchemaLazyDescription | SchemaFieldDescription;
    meta(): Record<string, unknown> | undefined;
    meta(obj: Record<string, unknown>): Lazy<T, TContext, TDefault, TFlags>;
}
export default Lazy;
