import type { AnyObject, InternalOptions, Callback, Message, Maybe } from './types';
import type Reference from './Reference';
import { Defined, Flags, NotNull, SetFlag, Thunk, ToggleDefault, ISchema, UnsetFlag } from './util/types';
import BaseSchema, { SchemaInnerTypeDescription, SchemaSpec } from './schema';
import { ResolveOptions } from './Condition';
export declare type RejectorFn = (value: any, index: number, array: any[]) => boolean;
export declare function create<C extends AnyObject = AnyObject, T = any>(type?: ISchema<T, C>): ArraySchema<T, C, undefined, "", T[] | undefined>;
export declare namespace create {
    var prototype: ArraySchema<any, any, any, any, any>;
}
export default class ArraySchema<T, TContext, TDefault = undefined, TFlags extends Flags = '', TIn extends any[] | null | undefined = T[] | undefined> extends BaseSchema<TIn, TContext, TDefault, TFlags> {
    innerType?: ISchema<T, TContext>;
    constructor(type?: ISchema<T, TContext>);
    protected _typeCheck(v: any): v is NonNullable<TIn>;
    private get _subType();
    protected _cast(_value: any, _opts: InternalOptions<TContext>): any;
    protected _validate(_value: any, options: InternalOptions<TContext> | undefined, callback: Callback): void;
    clone(spec?: SchemaSpec<any>): this;
    concat<TOther extends ArraySchema<any, any, any>>(schema: TOther): TOther;
    concat(schema: any): any;
    of<U>(schema: ISchema<U, TContext>): ArraySchema<U, TContext, TFlags>;
    length(length: number | Reference<number>, message?: Message<{
        length: number;
    }>): this;
    min(min: number | Reference<number>, message?: Message<{
        min: number;
    }>): this;
    max(max: number | Reference<number>, message?: Message<{
        max: number;
    }>): this;
    ensure(): ArraySchema<T, TContext, TIn, ToggleDefault<TFlags, TIn>, TIn>;
    compact(rejector?: RejectorFn): this;
    describe(options?: ResolveOptions<TContext>): SchemaInnerTypeDescription;
}
export default interface ArraySchema<T, TContext, TDefault = undefined, TFlags extends Flags = '', TIn extends any[] | null | undefined = T[] | undefined> extends BaseSchema<TIn, TContext, TDefault, TFlags> {
    default<D extends Maybe<TIn>>(def: Thunk<D>): ArraySchema<T, TContext, D, ToggleDefault<TFlags, D>, TIn>;
    concat<IT, IC, ID, IF extends Flags, IIn extends Maybe<IT[]>>(schema: ArraySchema<IT, IC, ID, IF, IIn>): ArraySchema<NonNullable<T> | IT, TContext & IC, Extract<IF, 'd'> extends never ? TDefault : ID, TFlags | IF, IIn>;
    concat(schema: this): this;
    defined(msg?: Message): ArraySchema<T, TContext, TDefault, TFlags, Defined<TIn>>;
    optional(): ArraySchema<T, TContext, TDefault, TFlags, TIn | undefined>;
    required(msg?: Message): ArraySchema<T, TContext, TDefault, TFlags, NonNullable<TIn>>;
    notRequired(): ArraySchema<T, TContext, TDefault, TFlags, Maybe<TIn>>;
    nullable(msg?: Message): ArraySchema<T, TContext, TDefault, TFlags, TIn | null>;
    nonNullable(): ArraySchema<T, TContext, TDefault, TFlags, NotNull<TIn>>;
    strip(enabled: false): ArraySchema<T, TContext, TDefault, UnsetFlag<TFlags, 's'>>;
    strip(enabled?: true): ArraySchema<T, TContext, TDefault, SetFlag<TFlags, 's'>>;
}
