import type Reference from '../Reference';
import type { Maybe, Optionals } from '../types';
import type { ISchema } from './types';
export declare type ObjectShape = {
    [k: string]: ISchema<any> | Reference;
};
export declare type AnyObject = {
    [k: string]: any;
};
export declare type TypeFromShape<S extends ObjectShape, C> = {
    [K in keyof S]: S[K] extends ISchema<any, C> ? S[K]['__outputType'] : unknown;
};
export declare type DefaultFromShape<Shape extends ObjectShape> = {
    [K in keyof Shape]: Shape[K] extends ISchema<any> ? Shape[K]['__default'] : undefined;
};
export declare type MergeObjectTypes<T extends Maybe<AnyObject>, U extends AnyObject> = ({
    [P in keyof T]: P extends keyof U ? U[P] : T[P];
} & U) | Optionals<T>;
export declare type PartialDeep<T> = T extends string | number | bigint | boolean | null | undefined | symbol | Date ? T | undefined : T extends Array<infer ArrayType> ? Array<PartialDeep<ArrayType>> : T extends ReadonlyArray<infer ArrayType> ? ReadonlyArray<ArrayType> : {
    [K in keyof T]?: PartialDeep<T[K]>;
};
declare type OptionalKeys<T extends {}> = {
    [k in keyof T]: undefined extends T[k] ? k : never;
}[keyof T];
declare type RequiredKeys<T extends object> = Exclude<keyof T, OptionalKeys<T>>;
export declare type MakePartial<T extends object> = {
    [k in OptionalKeys<T> as T[k] extends never ? never : k]?: T[k];
} & {
    [k in RequiredKeys<T> as T[k] extends never ? never : k]: T[k];
};
export {};
